#pragma once

#include <QList>
#include <QSharedPointer>
#include <QString>

#include <variant>

class QWidget;
enum class FileSelectState;
enum class FileType;
struct BonusPoints;
struct DynexiteExam;
struct ExamSave;
struct StudentData;

enum class FileOpenErrorCode {
    Success,
    OsError,
    CsvParseError,
    D2rParseError,
    UnknownError
};

struct FileOpenError {
    FileOpenErrorCode mEc = FileOpenErrorCode::Success;
    QString mDetails{};

    QList<QString> mWarnings{};
};

class InputModel {
public:
    /**
     * \brief Construct a model that controls file loading for the given file
     * type.
     */
    explicit InputModel(FileType t);

    /**
     * \brief Validates that the user selected a correct file and stores the
     * content in the appropriate data structure in `mLoadedData`.
     *
     * \param path Path to the file
     *
     * \return Success
     */
    FileOpenError loadCsv(const QString &path);

    /**
     * \brief Which file type this model stores.
     */
    const FileType mType;

    /**
     * \brief Moves ownership of the stored data to the caller, is empty
     * afterwards.
     * \return The stored data.
     */
    [[nodiscard]] auto get() { return std::move(mLoadedData); }
    [[nodiscard]] std::variant<DynexiteExam *, StudentData *, BonusPoints *,
                               ExamSave *>
    read() const;

private:
    std::variant<QSharedPointer<DynexiteExam>, QSharedPointer<StudentData>,
                 QSharedPointer<BonusPoints>, QSharedPointer<ExamSave>>
        mLoadedData{};
};

// Display a warning pop-up if too low an amount of students in the dynexite
// file are also in the RWTHonline file todo: noexcept?
void warnIfLowStudentIntersection(QWidget *parent,
                                  const InputModel &dynexiteModel,
                                  const InputModel &rwthModel,
                                  const InputModel &bonusModel,
                                  const FileSelectState &bonusState) noexcept;
