#include "heuristics.hpp"

#include "util/enums.hpp"

#include <QString>

#if !RANGE_JOIN_SUPPORTED
GradeBoundaryStepSize
guessGradeBoundaryStepSize(const QList<Points> &allPointValues) noexcept {

    if (allPointValues.empty()) {
        return GradeBoundaryStepSize::Min1_00;
    } else {
        const auto min = impl::minStepSize(allPointValues);

        // return Min0_25 for step sizes < 0.25
        if (min == 1.0)
            return GradeBoundaryStepSize::Min1_00;
        else if (min == 0.5)
            return GradeBoundaryStepSize::Min0_50;
        else
            return GradeBoundaryStepSize::Min0_25;
    }
}

[[nodiscard]] GradeBoundaryStepSize
guessGradeBoundaryStepSize(const DynexiteExam &exam) noexcept {
    // Create list with all point values (max points, closed earned points, open
    // earned points) from an exam.
    auto allPointValues = exam.mTaskMaxPoints;
    for (const auto &listOfLists :
         {exam.mTaskEarnedPoints[GradingType::Closed],
          exam.mTaskEarnedPoints[GradingType::Open]}) {
        for (const auto &l : listOfLists) {
            allPointValues.append(l);
        }
    }
    return guessGradeBoundaryStepSize(allPointValues);
}
#endif

FileType getApproximateFileType(const QString &s) noexcept {
    if (QString::compare(s, QString("STUDY_PROGRAMME"), Qt::CaseInsensitive) ==
        0) {
        return FileType::rwth;
    }
    if (QString::compare(s, QString("AttemptID"), Qt::CaseInsensitive) == 0) {
        return FileType::dynexite;
    }
    if (QString::compare(s, QString("Vorname"), Qt::CaseInsensitive) == 0) {
        return FileType::bonus;
    }
    return FileType::NR_VALS;
}
