#pragma once

#include "util/enums.hpp"
#include "view/gui/help_widget.hpp"

#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QString>
#include <QVBoxLayout>
#include <QWidget>

class CustomSlider;
class QDoubleSpinBox;
enum class GradeBoundaryStepSize;
struct FontSize;
struct GradeBoundary;

class MinMaxPointScrewColumn : public QVBoxLayout {
    Q_OBJECT

public:
    explicit MinMaxPointScrewColumn(GradingType grading,
                                    QWidget *parent = nullptr);

    void setWidgetsEnabled(bool enable) const;

private:
    QWidget *mParent;

    GradingType mGrading;

    QHBoxLayout *mHLayouts[4]{};
    QLabel *mDescriptions[2]{};
    QDoubleSpinBox *mSpinBoxes[4]{};
    CustomSlider *mSliders[2]{};

    HelpWidget *mQuestionIcon;

    void setSpinboxRange(int row, GradeBoundary min, GradeBoundary max,
                         GradeBoundary start,
                         GradeBoundaryStepSize minPointStep) const;
    void setSliderRange(int row, int sliderIdx, GradeBoundary rangeMin,
                        GradeBoundary rangeMax, GradeBoundary start,
                        GradeBoundary maxPoints) const;

    void createLabel(GradingType grading, int labelIdx, const QString &text,
                     int row, bool isMin, QWidget *parent);
    void createSlider(GradingType grading, int sliderIdx, int row,
                      QWidget *parent);

    void createQuestionIcon(GradingType grading, int row, bool isMin,
                            QWidget *parent);

private slots:
    void gradesChanged();
    void sliderChange(GradingType grading, int sliderIdx, int value);
    void spinboxChange(GradingType grading, int spinboxIdx, double value);
};

class MinMaxPointScrews : public QWidget {
    Q_OBJECT

public:
    explicit MinMaxPointScrews(QWidget *parent = nullptr);

    void setEnabled(GradingType type, bool enabled) const;

    void changeFontSize(FontSize newFontSize);

private slots:
    void gradesChanged();

    // private:
public:
    QVBoxLayout mVLayout{this};
    QHBoxLayout mHLayout{};
    QGridLayout mAvgLayout{};

    MinMaxPointScrewColumn *mClosedColumn = nullptr;
    MinMaxPointScrewColumn *mOpenColumn = nullptr;

    QLabel mAverageGrade{}, mAverageGradePassed{}, mNrPassed{};
    QLabel mAvgLabel, mAvgPassedLabel, mNrPassedLabel;

    void createScrewColumns();
    void createAvgLabels();
};
