#include "view_settings.hpp"

#include "data/data_manager.hpp"
#include "util/defaults.hpp"
#include "util/grade_boundary_step.hpp"

#include <QLabel>
#include <QSignalBlocker>

ViewSettingsWindow::ViewSettingsWindow(QWidget *parent)
    : QWidget(parent), mLayout(this), mCurrentBucketSize(1) {

    setWindowTitle("Settings");

    mPointHistogramBucketSize.setRange(defaults::minBucketSize,
                                       defaults::maxBucketSize);
    mPointHistogramBucketSize.setSingleStep(defaults::bucketStep);
    mPointHistogramBucketSize.setDecimals(2);

    mLayout.addWidget(new QLabel("Point histogram bucket size:", this));
    mLayout.addWidget(&mPointHistogramBucketSize);

    // receive notification from combo boxes
    connect(&DataManager::getInstance(), &DataManager::rawDataChanged, this,
            &ViewSettingsWindow::pointsChanged);
    connect(&mPointHistogramBucketSize, &QDoubleSpinBox::valueChanged, this,
            &ViewSettingsWindow::bucketSizeChanged);
}

void ViewSettingsWindow::pointsChanged() {
    const auto cur = DataManager::getCurrent();

    if (!cur)
        return;

    QSignalBlocker blocker(&mPointHistogramBucketSize);
    mCurrentBucketSize =
        as<double>(cur->mUserPointConfig.mGradeBoundaryStepSize);
    mPointHistogramBucketSize.setValue(mCurrentBucketSize);

    // Don't emit the "histogram bucket size changed" signal here, because
    // pointsChanged already implies it
}

void ViewSettingsWindow::bucketSizeChanged(const double d) {
    mCurrentBucketSize = d;

    emit pointHistoBucketSizeChanged(d);
}
